function A=randtril(N,d,varargin)
% FUNCTION fc_amat.random.randtril
% -- fc_amat.random.randtril(N,d)
%      Returns a N-by-d-by-d amat object whose matrices are lower triangular  
%      with non zeros elements drawn from the uniform distribution on open 
%      interval (a,b)=(0,1).
%      
% -- fc_amat.random.randtril(...,key,value)
%      Optional key/value pairs pairs arguments are available with keys:
%        - 'nc',  number of columns of matrices (default: d)
%        - 'k',  offset of k diagonals above or below the main diagonal; above 
%          for positive k and below for negative k.
%        - 'complex', if value is true the amat object is complex 
%                     (default false i.e real amat object)
%        - 'class', value can be 'single' or 'double' (default).
%        - 'a', to set a (lower bound of the interval) value (0 by default).
%        - 'b', to set b (upper bound of the interval) value (1 by default).
%
%      See also: fc_amat.random.randtriu, fc_amat.random.randdiag
%                class amat
%
  p = inputParser;
  p.addParamValue('nc',d,@isscalar); % number of columns
  p.addParamValue('k',0,@isscalar); % diag number
  p.addParamValue('class','double',@(x) ismember(lower(x),{'single','double'})); 
  p.addParamValue('complex',false,@islogical);
  p.addParamValue('a',0,@isscalar); 
  p.addParamValue('b',1,@isscalar); 
  
  p.parse(varargin{:});
  nc=p.Results.nc;k=p.Results.k;
  a=p.Results.a;b=p.Results.b;
  classname=p.Results.class;
  assert( a < b , 'a must be less than b!')
  h=b-a;
  if p.Results.complex
    A=complex(fc_amat.zeros(N,d,nc,classname));
    gen=@(e) complex(a+h*rand(N,e,classname),a+h*rand(N,e,classname));
  else
    A=fc_amat.zeros(N,d,nc,classname);
    gen=@(e) a+h*rand(N,e,classname);
  end
  for i=1:min(d,nc)
    e=min(i+k,nc);
    if ( e>=1 ), A(i,1:e)=gen(e); end
  end
end
